/*
 * Option.cc -- Definitions for the Option struct.  No longer nice and tiny.
 *
 * $Id: Option.cc,v 1.5 2000/01/18 22:34:35 devphil Exp $
*/

#include "go_decls.h"
#include <iostream>
#include "utils.h"
#include "outfile.h"


namespace {
    const char * const defaults[] = { "false", "0", "0.0", "" };
    const char * const helptexts[] = { "unused", "N", "N.N", "str" };
}  // end namespace


Option::
Option (const string& name_, char le_, const string& descrip_, Argument arg_)
    : letter (le_),
      argtype (arg_),
      descrip (descrip_),
      name (name_),
      exclusion_name (NULL),       // might not use this
      generate_given (false)       // or this (assume we don't)
{
    // anything?
}


void
Option::handle (const Extres& E)
{
    defaulttext = (E.default_string ? (*(E.default_string))
                                    : defaults[argtype]);
    generate_given = E.given;
    if (argtype == None)
        exclusion_name = E.argname_string;
    else
        argname = (E.argname_string ? (*(E.argname_string))
                                    : helptexts[argtype]);
}


void
Option::write_declaration (ostream& os, int indentation)
{
    os << spaces(indentation) << "// -" << letter << "  "
       << descrip << '\n' << spaces(indentation) << "extern ";

    string  s;
    int     i;
    switch (argtype) {
        case None:
            s = "bool  ";
            break;
        case Int:
            s = "int   ";
            break;
        case Real:
            s = "double  ";
            break;
        case String:
            s = "std::string  ";
            break;
        case Help:
        case Version:
            break;
    }
    i = s.length();
    os << s << genopt_symprefix_option << name << "_option;\n";

    if (generate_given) {
        os << spaces(indentation) << "extern bool" << spaces(i-4)
           << genopt_symprefix_option << name << "_option_given;\n";
    }
    os << '\n';
}


void
Option::write_definition (ostream& os, int indentationlvl)
{
    string  s;
    switch (argtype) {
        case None:
            s = "bool         ";
            break;
        case Int:
            s = "int          ";
            break;
        case Real:
            s = "double       ";
            break;
        case String:
            s = "std::string  ";
            break;
        case Help:
        case Version:
            break;
    }
    os << spaces(4*indentationlvl) << s
       << genopt_symprefix_option << name << "_option";

    // The only time defaulttext == "" is when a string is being used
    // and the user has not specified an initial value.
    if (!defaulttext.empty()) {
        os << " = ";
        if (argtype == String)  os << '\"';
        os << defaulttext;
        if (argtype == String)  os << '\"';
    }
    os << ";\n";

    if (generate_given) {
        os << spaces(4*indentationlvl) << "bool" << spaces(9)
           << genopt_symprefix_option << name
           << "_option_given = false;\n";
    }
    os << '\n';
}


string
Option::quoted_help_string()
{
    string ret = "\"  -";

    ret += letter;
    if (argtype != None) {
        ret += ' ';
        ret += argname;
    }

    rjust (ret, descrip);

    // maybe enclose this part in "if (E.default_string) {" -- have to
    // preserve that setting from the Extres, of course
    ret.append ("\"                                  [default ");
    if (argtype == String)  ret.append("\\\"");
    ret.append (defaulttext);
    if (argtype == String)  ret.append("\\\"");
    ret.append ("]\\n\"\n");

    return ret;
}


string
Option::usage_string()
{
    string ret = "[-";
    ret += letter;
    // testing is probably faster than appending an empty string
    if (argtype != None) {
        ret += ' ';
        ret += argname;
    }
    return ret += "] ";
}


string
Option::assign_from_optarg_string()
{
    string ret;
    switch (argtype) {
        case None:
            ret = string("!") + genopt_symprefix_option + name + "_option;\n";
            break;
        case Int:
            ret = "atoi(optarg);\n";
            break;
        case Real:
            ret = "atof(optarg);\n";
            break;
        case String:
            ret = "optarg;\n";
            break;
        case Help:
        case Version:
            break;
    }
    return ret;
}


