/*
 * go_decls.h -- Declarations of functions and variables.  Not much
 *               else to say, is there?
 *
 * $Id: go_decls.h,v 1.8 2000/01/18 22:34:35 devphil Exp $
*/

#ifndef GO_DECLS_H
#define GO_DECLS_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <iosfwd>
#include <list>
#include <string>
using std::string;


// The spelling is intentional.  Pronounce it like a stereotypical West
// Texas accent (EX-trees), as in "that thar's a lot uh them extrees."  I
// needed a name that would be easily distinguishable.  Deal with it.
struct Extres
{
    // Currently not deallocating any of the free store here.
    bool           given;
    std::string   *default_string,
                  *argname_string;

    Extres () : given (false), default_string (NULL), argname_string(NULL)
                { ; }
};


struct Option
{
    enum Argument
    {
        None = 0,        // standalone flags
        Int,         // these three are self-explanatory
        Real,
        String,
        Help,        // these last two are special cases
        Version
    };

    char          letter;
    Argument      argtype;        // what kind of arg, if any (this is
                                  // far easier and more efficient than
                                  // using derivation)
    string        argname;        // holds the 'help' name, if any
    string        descrip;
    string        name;           // for <prefix>_<name>_option{_given}
    string        defaulttext;    // source to emit as initialization

    // these are extras and not always used
    string*       exclusion_name;       // flag
    bool          generate_given;       // everything_else

    // name, letter, descrip, type
    Option (const string&, char, const string&, Argument);

    // each type takes what it needs out of the parameter
    void handle (const Extres&);

    // emits code for the header file at a certain indentation level
    void write_declaration (ostream&, int);

    // emits code for the body file at a certain indentation level
    void write_definition (ostream&, int);

    // returns " -x         Does foo\n" string for print_help()
    string quoted_help_string();

    // returns [-x argfoo] string for print_usage()
    string usage_string();

    // returns everything after the = for the assignment inside the getopt
    // switch statement, including the newline
    string assign_from_optarg_string();
};


typedef  std::list<Option*>   Optionlist;

// The scanner is going to allocate and return pointers-to-string, so
// we'll just use those rather than copying them.  Most of these are
// defined in vars.cc.
extern string *client_packagename;
extern string *client_version;
extern int     remaining_args_required;

extern string  errorstring;
extern bool    help_requested, version_requested;

extern Optionlist   allopts;    // defined in go_parse.y


#endif /* GO_DECLS_H */

