%{
/*
 * go_parse.y -- Parser generator for genopt.  Requires bison.
 *
 * $Id: go_parse.y,v 1.6 2000/01/06 20:07:25 devphil Exp $
*/


#include <string>
#include <stdlib.h>
#include <stdio.h>
#include "go_decls.h"

void yyerror(char const *);

extern int yylex (void);
extern int lineno;

Optionlist   allopts;   // contains all 'optentry' lines from the input

namespace {
    bool string_icmp (const string&, const string&);  // true if equal

    Extres   *extra_collection;
}

%}

%union {
    /* These can come from parsing actions. */
    Option       *an_option;
    Extres       *some_extras;

    /* These can also come directly from the scanner. */
    std::string  *str;
    int           intval;
    char          charval;
}

/* These tokens make use of yylval and therefore need to be
 * "typed" accordingly.
*/
%token <intval>   INTEGER  OPTTYPE
%token <str>      STRING  WORD
%token <charval>  CHARACTER

/* These are the main keywords. */
%token PACKAGE  VERSION  OPTENTRY  OPTTYPE

/* These are the optional keywords. */
%token REQUIRED

/* These are the option-extras. */
%token DEFAULT  EXCLUDE  GIVEN  ARGNAME

%type <an_option>    option

%start optionfile

%%

optionfile:  PACKAGE  STRING  VERSION  STRING  others  optionlist
    {
         client_packagename = $2;
         client_version = $4;
    }
    ;


others:  /* may be empty */   { ; }
    |  others  REQUIRED  INTEGER
        {
             remaining_args_required = $3;
        }
    ;


/* This seems utterly cheesy. */
optionlist:  option              { allopts.push_back($1); }
          |  optionlist  option  { allopts.push_back($2); }
    ;


option:  OPTENTRY  WORD  OPTTYPE  CHARACTER  STRING  prep_extras  extras
    {
         // this cast is safe... I suppose I could've put an O::A in YYSTYPE
         Option::Argument  type = static_cast<Option::Argument>($3);

         Option *o = new Option (*($2), $4, *($5), type);
         o->handle (*extra_collection);
         delete extra_collection;  // don't need to leak memory THAT badly...
         extra_collection = NULL;
         $$ = o;
    }
    ;


prep_extras:
    /* empty placeholder for action */   { extra_collection = new Extres; }


extras:  /* may be empty */   { ; }
    |  extras  DEFAULT  STRING
        {
             extra_collection->default_string = $3;
        }
    |  extras  EXCLUDE  WORD
        {
             extra_collection->argname_string = $3;
        }
    |  extras  GIVEN
        {
             extra_collection->given = true;
        }
    |  extras  ARGNAME  STRING
        {
             extra_collection->argname_string = $3;
        }
    ;


%%

void yyerror(char const *msg)
{
    extern char*  yytext;
    fprintf (stderr, "Error (%s) on line %d:  %s\n", msg, lineno, yytext);
}


namespace {  // indenting violated
bool string_icmp (const string& lhs, const string& rhs)
{
    string  lower_lhs (lhs),
            lower_rhs (rhs);
    string::size_type i;

    for (i = 0; i < lower_lhs.length(); ++i)
        lower_lhs = tolower(lower_lhs[i]);
    for (i = 0; i < lower_rhs.length(); ++i)
        lower_rhs = tolower(lower_rhs[i]);
    return lower_lhs == lower_rhs;
}
}  // end namespace

