%{
/*
 * go_scan.l -- Simple utility scanner.  Requires flex.
 *
 * $Id: go_scan.l,v 1.3 1999/12/30 18:27:37 devphil Exp $
*/


#include <stdlib.h>     /* for atoi */
#include <stdio.h>
#include <string>

#ifndef GO_DECLS_H
#  include <go_decls.h>
#endif

#ifndef GO_PARSE_H
#  define GO_PARSE_H
#  include "go_parse.h"
#endif

extern int lineno;

%}

ws           [ \t\r]*
eol          \n
comment      #.*
character    '[A-Za-z0-9]'
word         [A-Za-z0-9_]*
qstring      \"[^\"\n]*[\"\n]
intnum       -?(([0-9]+)([eE][-+]?[0-9]+)?)

%%

package    { return PACKAGE; }
packageversion  { return VERSION; }
option     { return OPTENTRY; }
required   { return REQUIRED; }

default    { return DEFAULT; }
exclude    { return EXCLUDE; }
given      { return GIVEN; }
argname    { return ARGNAME; }

string     { yylval.intval = Option::String; return OPTTYPE; }
integer    { yylval.intval = Option::Int; return OPTTYPE; }
real       { yylval.intval = Option::Real; return OPTTYPE; }
flag       { yylval.intval = Option::None; return OPTTYPE; }
help       { yylval.intval = Option::Help; return OPTTYPE; }
version    { yylval.intval = Option::Version; return OPTTYPE; }


{intnum} {
              yylval.intval = atoi (yytext);
              return INTEGER;
         }

{qstring} {
               if (yytext[yyleng-1] != '"') {
                   fprintf (stderr, "Warning:  Unterminated string on "
                            "line %d; I was removing quotes anyhow.", lineno);
               }
               yylval.str = new string (yytext+1, yyleng-2);
               return STRING;
          }

{word} {
            yylval.str = new string (yytext);
            return WORD;
       }

{character} {
                 yylval.charval = yytext[1];  /* lose the surrounding '' */
                 return CHARACTER;
            }

{eol}          { ++lineno; }
{ws}           /* Ignore whitespace */ ;
{comment}      /* Ignore comments */ ;
.   { return yytext[0];  /* Pass back anything unrecognized */
}

%%

