/*
 * header_file.cc -- Routines to generate the header file.
 *
 * $Id: header_file.cc,v 1.1 1999/12/30 00:55:10 devphil Exp $
*/

#include "go_decls.h"
#include "utils.h"
#include "filegenfuncs.h"
#include "outfile.h"
#include <fstream>

using std::string;

namespace {
    bool  any_strings_being_used ();
}


bool
write_header_file ()
{
    string         filename (genopt_outfile_option+".h");
    std::ofstream  of (filename.c_str());
    int            indention = genopt_namespace_option_given ? 4 : 0;

    if (!of) {
        errorstring = "Couldn't create output file " + filename;
        return false;
    }

    // write out #includes and the like
    of << "/* \n"
          " * " << (*client_packagename) << ' ' << (*client_version) << "\n"
          " * " << filename << " -- Declarations of command-line options.\n"
          " *\n"
          " * GENERATED BY GENOPT -- CHANGES WILL BE LOST\n"
          "*/\n"
          "\n"
          "#ifndef " << preprocsymbol(filename) << "\n"
          "#define " << preprocsymbol(filename) << "\n"
          "\n";

    if (any_strings_being_used())
        of << "#include <string>\n";

    of << std::endl;

    // if a namespace is requested, open it
    if (genopt_namespace_option_given)
        of << "namespace " << genopt_namespace_option << " {\n\n";

    // tell each of the Options to write out its declaration
    for (Optionlist::iterator i = allopts.begin(); i != allopts.end(); ++i) {
        (*i)->write_declaration(of,indention);
    }

    // write out processing-function declaration
    of << "\n"
       << spaces(indention)
       << "extern int "
       << genopt_symprefix_option
       << "process_options (int, char**);\n"
       << std::endl;

    // if a namespace is requested, close it
    if (genopt_namespace_option_given)
        of << "}  // end namespace\n";

    of << "\n#endif /* " << preprocsymbol(filename) << " */\n\n";
    of.close();
    if (!of) {
        errorstring = "Couldn't close output file " + filename;
        return false;
    }
    return true;
}




namespace {

bool
any_strings_being_used ()
{
    bool ret = false;

    Optionlist::const_iterator  i = allopts.begin();
    while ((!ret) && (i != allopts.end())) {
        ret = (*i)->argtype == Option::String;
        ++i;
    }

    return ret;
}


}  // end namespace


