/*
 * source_file.cc -- Routines to generate the source body file.  This is
 *                   a freakin' nightmare.  The write_source_file() function
 *                   is huge, but splitting it up wouldn't gain much, since
 *                   none of the resulting pieces would be reused anywhere.
 *                   (This will remain the case until C code can be gen'd.)
 *
 * $Id: source_file.cc,v 1.4 2000/01/19 17:25:32 devphil Exp $
*/

#include "go_decls.h"
#include "utils.h"
#include "filegenfuncs.h"
#include "outfile.h"
#include <fstream>



using std::string;

namespace {
    string getopt_arg_string ();
}


// spaces per tab  -- ugly; I should be shot for this
#define TS  (spaces(4*ilvl))

bool
write_source_file ()
{
    string         filename (genopt_outfile_option+".cc");
    std::ofstream  of (filename.c_str());
    int            ilvl = 0;   // indentation level

    Optionlist::iterator oit;

    if (!of) {
        errorstring = "Couldn't create output file " + filename;
        return false;
    }

    // You know, after this point there's a remarkable lack of error checking.

    // write out #includes and the like
    of << "/* \n"
          " * " << (*client_packagename) << ' ' << (*client_version) << "\n"
          " * " << filename << " -- Definitions to perform command-line "
                               "option processing.\n"
          " *\n"
          " * GENERATED BY GENOPT -- CHANGES WILL BE LOST\n"
          "*/\n"
          "\n"
          "#include <stdlib.h>    // eventually <cstdlib>\n"
          "#include \"" << genopt_outfile_option << ".h\"\n"
          "#include <string>\n"
          "#include <iostream>\n"
          "#include <libgen.h>    // or whatever contains basename(3)\n"
          "\nusing std::string;\n";

    // write out declarations of the file-local functions
    of << "\nnamespace {\n"
          "    // name of program; this is guessed and may need"
                                  " to be changed by you\n"
          "    string  argvzero;\n\n"
          "    void print_usage();\n"
          "    void print_help();\n"
          "    void print_version();\n"
          "}  // end unnamed namespace\n\n\n";

    // if a namespace is requested, open it
    if (genopt_namespace_option_given) {
        of << "namespace " << genopt_namespace_option << " {\n\n";
        ++ilvl;
    }

    // tell each of the Options to write out its definition, along with
    // any specified initializations
    for (oit = allopts.begin(); oit != allopts.end(); ++oit) {
        (*oit)->write_definition(of,ilvl);
    }

    // write out processing-function definition
    of << "\n"
       << TS << "int "
       << genopt_symprefix_option
       << "process_options (int argc, char** argv)\n"
       << TS << "{\n";
    ++ilvl;
    of << TS << "argvzero = basename(argv[0]);  // possibly a Bad Thing to do\n"
       << TS << "opterr = 0;\n\n"
       << TS << "int c;\n"
       << TS << "while ((c = getopt(argc, argv, \""
       << getopt_arg_string()
       << "\")) != EOF) {\n";
    ++ilvl;
    of << TS << "switch (c) {\n";
    ++ilvl;

    // write out the case statements
    for (oit = allopts.begin(); oit != allopts.end(); ++oit) {
        Option &ref = **oit;
        of << TS << "case \'" << ref.letter << "\':\n";
        ++ilvl;
        if ((ref.argtype != Option::None) && ref.generate_given)
            of << TS << genopt_symprefix_option << ref.name
                     << "_option_given = true;\n";
        of << TS << genopt_symprefix_option << ref.name << "_option = "
                 << ref.assign_from_optarg_string();
        if ((ref.argtype == Option::None) && ref.exclusion_name)
            of << TS << genopt_symprefix_option << (*ref.exclusion_name)
                     << "_option = !"
                     << genopt_symprefix_option << (*ref.exclusion_name)
                     << "_option;\n";
        of << TS << "break;\n";
        --ilvl;
    }

    // unrecognized options
    of << TS << "case '?':\n";
    ++ilvl;
    of << TS << "std::cerr << argvzero"
       << " << \":  Unrecognized or missing option \\'\"\n"
       << TS << "          << static_cast<char>(optopt) << \"\\'\\n\";\n"
       << TS << "print_usage();\n"
       << TS << "::exit(1);\n"
       << TS << "break; /* NOTREACHED */\n";
    --ilvl;

    // temporary help option
    of << TS << "case 'h':\n";
    ++ilvl;
    of << TS << "print_help();\n"
       << TS << "::exit(0);\n"
       << TS << "break; /* NOTREACHED */\n";
    --ilvl;

    // temporary version option
    of << TS << "case 'v':\n";
    ++ilvl;
    of << TS << "print_version();\n"
       << TS << "::exit(0);\n"
       << TS << "break; /* NOTREACHED */\n";
    --ilvl;
    // other cases ?
    --ilvl;
    of << TS << "}\n";   // end the switch
    --ilvl;
    of << TS << "}\n";   // end the while

    if (remaining_args_required > 0) {
        of << TS << "// test for remaining_args_required\n"
           << TS << "if (argc - optind < 1) {\n";
        ++ilvl;
        of << TS << "std::cerr << argvzero"
           << " << \":  Required argument missing\\n\";\n"
           << TS << "print_usage();\n"
           << TS << "::exit(1);\n";
        --ilvl;
        of << TS << "}\n";
    }

    of << TS << "return optind;\n";
    --ilvl;
    of << TS << "}\n";   // end the function

    // if a namespace is requested, close it
    if (genopt_namespace_option_given) {
        of << "}  // end user-defined namespace\n\n";
        --ilvl;
    }

    // write out definitions of the file-local functions
    ilvl = 1;
    of << "\nnamespace {"
          "   // Indentation violated inside this namespace block\n"
          "\nvoid print_usage()\n{\n"
       << TS << "using std::cerr;\n"
       << TS << "cerr << \"Usage:  \" << argvzero\n"
       << TS << "     << \" [-hv] \"     // assuming you want help/version"
                ", else delete/comment\n"
       << TS << '\"';
    {
        string S;
        for (oit = allopts.begin(); oit != allopts.end(); ++oit) {
            S.append((*oit)->usage_string());
        }
        //linebreak (S, 8, 20);    // "Usage:  " == 8 chars
        of << S << '\"';
    }
    of << '\n' << TS << "     << std::endl;\n}\n\n";

    // print_help
    of << "\nvoid print_help()\n{\n"
       << TS << "using std::cout;\n\n"
       << TS << "print_version();\n"
       << TS << "print_usage();\n"
       << TS << "// Lines with [default] can be removed entirely if you like\n"
       << TS << "cout << \"Options:\\n\" <<\n";
    for (oit = allopts.begin(); oit != allopts.end(); ++oit) {
        of << (*oit)->quoted_help_string();
    }
    of << TS << "     << std::endl;\n}\n\n";

    // print_version
    of << "\nvoid print_version()\n{\n"
       << TS << "std::cout << \""
       << (*client_packagename)
       << " version "
       << (*client_version)
       << "\" << std::endl;\n"
       << TS << "//std::cout << \"v"
       << (*client_version)
       << "\" << std::endl;\n"
       << "}\n\n";

    of << "}  // end unnamed namespace\n";

    of << "\n\n";
    of.close();
    if (!of) {
        errorstring = "Couldn't close output file " + filename;
        return false;
    }
    return true;
}
#undef TS




namespace {

string getopt_arg_string()
{
    string r = "hv";
    for (Optionlist::iterator i = allopts.begin(); i != allopts.end(); ++i) {
        Option &ref = **i;
        r += ref.letter;
        if (ref.argtype != Option::None) {
            r += ':';
        }
    }
    return r;
}



}  // end namespace


