/*
 * utils.cc -- A bunch of string-generating functions and the like.
 *
 * $Id: utils.cc,v 1.3 2000/01/18 22:34:35 devphil Exp $
*/

#include "go_decls.h"
#include "utils.h"
#include <iostream>
#include <algorithm>
#include <iterator>

// need to learn locales
#include <ctype.h>


//  preprocsymbol("go_outfile.h") returns "GO_OUTFILE_H"
char spectoupper (char c)
{
    if (c == '.')  return '_';
    return toupper(c);
}

string
preprocsymbol (const string& i)
{
    string ret;

    ret.resize(i.size());
    std::transform (i.begin(), i.end(), ret.begin(), spectoupper);

    return ret;
}


//  rjust(str,text) appends enough spaces to 'str' such that 'text' starts
//  on (1-based) index 32, and then appends 'text' (if it wraps it will
//  also be indented to 32.  Explicit newlines and quotes are added (this
//  work is shared with Option::quoted_help_string).
void
rjust (string& str, const string& text)
{
    string::size_type L = text.length(),
                      already = str.length(),
                      pos = 0,
                      s,                 // amount of space to add
                      n,                 // amount of 'text' to add
                      tmpn;

    s = (already > 29) ? 5 : (31 - already);
    n = std::min((string::size_type)(78 - (already + s)), L);
    if (L > 47) {
        tmpn = n;
        while (tmpn && (!isspace(text[pos+tmpn])))  --tmpn;
        // if there are no spaces (!?!!), just print as much as possible
        n = (tmpn ? tmpn : n);
    }
    str.append (s, ' ');
    str.append (text, pos, n);
    str.append ("\\n\"\n");
    pos += n;
    L -= n;

    while (L) {
        n = std::min((string::size_type)47, L);
        if (L > 47) {
            tmpn = n;
            while (tmpn && (!isspace(text[pos+tmpn])))  --tmpn;
            // if there are no spaces (!?!!), just print as much as possible
            n = (tmpn ? tmpn : n);
        }

        str.append ("\"                             ");
        //str.append (1, '\"');
        //str.append (30, ' ');
        str.append (text, pos, n);
        str.append ("\\n\"\n");

        pos += n;
        L -= n;
    }
}


//  linebreak(str,indent,initialindent) replaces spaces with newlines to
//  break up 'str'.  After each newline 'indent' spaces will be inserted.
//  'initialindent' characters are assumed to be in 'str' already, and
//  those will not be broken up.
// XXX BROKEN AS HELL -- DO NOT USE -- NOT IMPORTANT
void
linebreak (string& str, string::size_type I, string::size_type II)
{
    const string::size_type pl = 78;    // per line

    string::size_type strl = str.length(),
                      pos = 0,
                      n;

    // first time needs to not do I and take II into account
    // N.B. that II characters might not actually be in str, we just
    // pretend that they are
    if (strl > pl) {
        n = pl - II;
        while ((n>II) && (!isspace(str[n])))  --n;
        if (isspace(str[n])) {
            str[n] = '\n';
            pos = n + 1;
        } else {
            // no spaces anywhere after the initial indent, screw it
            str.insert (pl - 1, 1, '\n');
            pos = pl - 1;
        }

        while (strl > pl) {
            str.insert (pos, I, ' ');
            strl += I;
            n = pos + pl;
            while ((n>pos+I) && (!isspace(str[n])))  --n;
            if (isspace(str[n])) {
                str[n] = '\n';
            } else {
                // no spaces anywhere after the indent, screw it
                str.insert (pos + pl - 1, 1, '\n');
            }
            pos += pl;
            strl -= pl;
        }
    }
}



